# -*- coding: utf-8 -*-
"""
colors.py
=========

A set of colors and color maps.

"""

import numpy as np

import brewer2mpl
from matplotlib import cm

# ColorBrewer by Drs. Cynthia Brewer and Mark Harrower of Pennsylvania
# State University. For more information on ColorBrewer, see:
# - Flash-based interactive map:
# http://colorbrewer2.org/
# - A quick visual reference to every ColorBrewer scale:
# http://bl.ocks.org/mbostock/5577023

# ColorBrewer scale 'Qualitative.Set1'.
# This one has nice "traditional" colors like reds and blues
brewer_set1 = brewer2mpl.get_map('Set1', 'qualitative', 9).mpl_colors

# ColorBrewer scale 'Qualitative.Set2'.
# This one are lesssaturated than those colors in Set1.
brewer_set2 = brewer2mpl.get_map('Set2', 'qualitative', 8).mpl_colors

# ColorBrewer scale 'Qualitative.Accent'.
brewer_accent = brewer2mpl.get_map('Accent', 'qualitative', 8).mpl_colors

# ColorBrewer scale 'Qualitative.Dark2'.
brewer_dark2 = brewer2mpl.get_map('Dark2', 'qualitative', 8).mpl_colors

# ColorBrewer scale 'Qualitative.Paired'.
brewer_paired = brewer2mpl.get_map('Paired', 'qualitative', 12).mpl_colors

# Set some commonly used colors
almost_black = '#262626'
light_grey = np.array([float(248) / float(255)] * 3)

# Color maps
reds = cm.Reds
reds.set_bad('white')
reds.set_under('white')

blues_r = cm.Blues_r
blues_r.set_bad('white')
blues_r.set_under('white')

# Need to 'reverse' red to blue so that blue=cold=small numbers,
# and red=hot=large numbers with '_r' suffix
brewer_blue_red = brewer2mpl.get_map('RdBu', 'Diverging', 11,
                                     reverse=True).mpl_colormap




# -*- coding: utf-8 -*-
"""
general.py
==========

General toolkits.

"""

from functools import wraps
import matplotlib as mpl

__all__ = ['MPLdecorator',]


class MPLdecorator:
     """
     An :class:`MPLdecorator` instance represents a specific context-decorator
     that makes it possible to configure matplotlib parameters
     without affecting matplotlib.pyplot.

     Parameters
     ----------
     :param rcp: matplotlib parameter settings
     :type rcp: dict

     Attributes
     ----------
     :param rcParams: matplotlib parameter settings
     :type rcParams: dict
     :param mpl_contexts: matplotlib rc contexts
     :type mpl_contexts: list of matplotlib.rc_context

     """
     mpl_contexts = []

     def __init__(self, rcp):
          self.rcParams = rcp

     def __call__(self, func):
          @wraps(func)
          def wrapper(*args, **kwargs):
               with self:
                    return func(*args, **kwargs)
          return wrapper

     def __enter__(self):
          context = mpl.rc_context(rc=self.rcParams)
          self.mpl_contexts.append(context)
          return context.__enter__()

     def __exit__(self, *args):
          return self.mpl_contexts.pop().__exit__(*args)



# -*- coding: utf-8 -*-
"""
presentation.py
===============

Context decorator for producing figures suitable in presentation for
Mac OS Keynote or Microsoft PowerPoint.

PDF format for image file are used, because it correctly uses sans-serif fonts.

"""

from .general import MPLdecorator
from .layout import point2inch, GOLDEN_RATIO
from .colors import brewer_set1

__all__ = ['presentation_decorator', ]

# Constants for presentation
_width_full_pt = 1024  # Units in number of points (or dots)
_width_normal_pt = int(_width_full_pt / 3.0)
width_normal = point2inch(_width_normal_pt)
width_tiny = 0.5 * width_normal
width_small = 0.8 * width_normal
width_large = 1.2 * width_normal
width_large2 = 1.5 * width_normal
width_huge = 1.8 * width_normal
width_full = 0.9 * point2inch(_width_full_pt)  # use 90% of presentation page.

# Default ratio for a single plot figure
# Golden ratio
height_width_ratio = GOLDEN_RATIO  # = height / width

_width = width_normal
_height = _width * height_width_ratio

_params = {'font.family' : 'sans-serif',
           'font.serif' : ['Times', 'Computer Modern Roman'],
           'font.sans-serif' : ['Helvetica', 'Arial', 'Lucida Grande'],
           'font.size' : 12,
           'font.weight' : 'normal',
           'text.usetex' : True,

           'axes.color_cycle': brewer_set1,
           'axes.labelsize' : 'medium',
           'axes.labelweight' : 'normal',
           'axes.linewidth' : 1.5,

           'figure.figsize' : (_width, _height),
           'figure.subplot.left' : 0.125,
           'figure.subplot.right' : 0.95,
           'figure.subplot.bottom' : 0.1,
           'figure.subplot.top' : 0.95,

           'savefig.dpi' : 300,
           'savefig.format': 'pdf',
           #'savefig.bbox': 'tight',
           # this will crop white spaces around images that will make
           # width/height no longer the same as the specified one.

           'legend.fontsize' : 'small',
           'legend.frameon' : False,
           'legend.numpoints' : 1,
           'legend.handlelength' : 2,
           'legend.scatterpoints' : 1,
           'legend.labelspacing' : 0.5,
           'legend.markerscale' : 0.9,
           'legend.handletextpad' : 0.5,  # pad between handle and text
           'legend.borderaxespad' : 0.5,  # pad between legend and axes
           'legend.borderpad' : 0.5,  # pad between legend and legend content
           'legend.columnspacing' : 1,  # pad between each legend column

           'text.fontsize' : 'medium',

           'xtick.major.size' : 6,
           #'xtick.minor.size' : 2,
           'xtick.major.width' : 1.5,
           #'xtick.minor.width' : 0.5,
           #'xtick.major.pad' : 4,
           #'xtick.minor.pad' : 4,
           #'xtick.color' : k,
           'xtick.labelsize' : 'medium',
           #'xtick.direction' : 'in',

           'ytick.major.size' : 6,
           #'ytick.minor.size' : 2,
           'ytick.major.width' : 1.5,
           #'ytick.minor.width' : 0.5,
           #'ytick.major.pad' : 4,
           #'ytick.minor.pad' : 4,
           #'ytick.color' : k,
           'ytick.labelsize' : 'medium',
           #'ytick.direction' : 'in',

           'lines.linewidth' : 1.5,
           'lines.markersize' : 6,
           #'lines.markeredgewidth' : 0,
           # 0 will make line-type markers, such as '+', 'x', invisible
          }

presentation_decorator = MPLdecorator(_params)




# -*- coding: utf-8 -*-
"""
styles.py
=========

Toolkits and constants for plotting styles.

"""

import itertools

from .colors import almost_black, brewer_set1

__all__ = ['lines', 'markers', 'nextlinestyle', 'linestyle_generator']


_colors = itertools.cycle(brewer_set1)

_line_styles = ['-', '--', '-.', ':']
lines = itertools.cycle(_line_styles)

_markers = ['o', 's', 'v', '^', 'D', 'p', 'h', '<', '>']
markers = itertools.cycle(_markers)

# for making hollow markers, use it together with marker_types
_markersh = ['o', 'o', 's', 's', 'v', 'v', '^', '^', 'D', 'D']
markersh = itertools.cycle(_markersh)

_marker_types = [False, True]  # True for hollow markers
marker_types = itertools.cycle(_marker_types)

def nextlinestyle(no_line=False, is_line=True, is_marker=True, is_hollow=True):
     """
     Generate a dict for configuring plot line style.

     NOTE: this function has a serious problem.
     As long as mpltex is imported,
     the beginning of the sequence cannot be configured later.
     This function is obsolete, please use linestyle_generator instead.

     The default line style is markers linked by lines, both styles are cycled,
     and hollow markers are included.
     Disable cycling one style by set the corresponding flag to False.

     :param no_line: if True, only markers, no line segments
     :type no_line: bool
     :param is_line: if True, cycle line styles. No effect when no_line=True.
     :type is_line: bool
     :param is_marker: if True, cycle markers.
     :type is_marker: bool
     :param is_hollow: if True, cycle markers including hollow styles.
     :param is_hollow: bool
     :return: dict of parameters of linestyle
     :rtype: dict
     """
     color = _colors.next()
     linestyle = lines.next()
     if(is_hollow):
          marker = markersh.next()
     else:
          marker = markers.next()
     marker_type = marker_types.next()
     if marker_type:  # hollow mark
          mew = 1
          mec = color
          mfc = 'None'
     else:
          mew = 1
          mec = color
          mfc = color

     linestyles = {}
     if (is_line):
          linestyles['linestyle'] = linestyle
     if (no_line):  # overwrite is_line if no_line is True
          linestyles['linestyle'] = ''
     if (is_marker):
          linestyles['marker'] = marker
          linestyles['mew'] = 0  # make the marker no edge
     if (is_hollow):
          linestyles['color'] = color
          linestyles['marker'] = marker
          linestyles['mew'] = mew
          linestyles['mec'] = mec
          linestyles['mfc'] = mfc
     return linestyles


def linestyle_generator(colors=brewer_set1, lines=_line_styles,
                        markers=_markers, hollow_styles=_marker_types):
     """
     Generate a dict for configuring plot line styles.

     The default line style is markers linked by lines, both styles are cycled,
     and hollow markers are included.

     Usage:
          linestyles = linestyle_generator()
          linestyle = linestyles.next()

     To exclude hollow markers, just let hollow_styles = [None].
     Note that all inputs should be iterable objects.

     :param colors: list of colors (tuple of RGB or #FFFFFF
                    or color name string)
     :type colors: list of tuple or list of string
     :param lines: list of line styles ('-', '--', '-.', ':')
     :type is_line: list of string
     :param markers: list of markers (see matplot for available markers)
     :type marker: list of charaters
     :param hollow_styles: True for hollow markers, False for filled markers.
     :param is_hollow: list of bool
     :return: dict of parameters of linestyle
     :rtype: dict
     """
     color_cycle = itertools.cycle(colors)
     line_cycle = itertools.cycle(lines)
     full_markers = itertools.product(markers, hollow_styles)
     marker_cycle = itertools.cycle(full_markers)
     while True:
        color = color_cycle.next()
        linestyle = line_cycle.next()
        marker, hollow = marker_cycle.next()
        if hollow is None:  # don't cycle hollow markers
            mew = 0
            mec = 'None'
            mfc = color
        elif hollow:  # make hollow markers
            mew = 1
            mec = color
            mfc = 'None'
        else:  # make filled markers
            mew = 1
            mec = color
            mfc = color
        yield {'linestyle':linestyle, 'marker':marker, 'mew':mew, 'mec':mec, 'mfc':mfc}



"""
mpltex
======

**mpltex** is a python package for producing publication quality images using matplotlib.
Inspired by `Olga Botvinnik <http://olgabotvinnik.com/>`_'s python package `prettyplotlib <https://github.com/olgabot/prettyplotlib>`_.

The internal matplotlib color cycle is replaced by ColorBrewer Set1 scale which looks less saturated and more pleasing to eyes.
For more information on ColorBrewer, see `a flash-based interactive map <http://colorbrewer2.org/>`_ and `a quick visual reference to all ColorBrewer scales <http://bl.ocks.org/mbostock/5577023>`_.

**mpltex** also enable cycle line styles and a selected set of line markers.
Hollow markers are supported.

Quickstart
----------

1. Install
^^^^^^^^^^

::

    $ pip install mpltex

**Required Packages**

-  `matplotlib <http://matplotlib.org/>`_. Can be installed via
   ``pip install matplotlib``.
-  `brewer2mpl <https://github.com/jiffyclub/brewer2mpl>`_. Can be
   installed via ``pip install brewer2mpl``.

2. Usage
^^^^^^^^

Just add one of **mpltex** decorators before your plot functions.

.. code:: python

    import mpltex

    @mpltex.acs_decorator
    def your_plot():
        # plot images by matplotlib ...

        # Save the image. Give a file name without extension.
        # You can also save figure outside your_plot if you like.
        fig.save_fig('/path/to/save/fig/figname')

    # Then use your_plot in a normal way.
    your_plot()

And it will produce images suitable for publishing in American Chemical Society (ACS).

**mpltex** also includes several helper functions to faciliate production of specific type of images.
Following codes will produce a set of line arts with cycled line styles and line markers with the help of ``mpltex.linestyle_generator`` function.

.. code:: python

    import matplotlib.pyplot as plt
    import mpltex

    @mpltex.acs_decorator
    def your_plot():
        # generate data x and y
        fig, ax = plt.subplots(111)
        for i in range(number_of_lines):
            # The default line style is iterating over
            # color, line, and marker with hollow types.
            linestyles = mpltex.linestyle_generator()
            ax.plot(x[i], y[i], label=str(i), **linestyles.next())

        ax.locator_params(nbins=5)  # limit the number of major ticks
        ax.legend(loc='best')  # show legend in a best location
        fig.tight_layout(pad=0.1)  # make layout as tight as possible
        fig.savefig('/path/to/save/fig/figname')

**Available Decorators**

* ``mpltex.acs_decorator``: output EPS images for publishing in ACS.
* ``mpltex.presentation_decorator``: output PDF images for presentation slides (Keynote).
* ``mpltex.web_decorator``: output PNG images for web pages.

Contribute
----------

Please do not hesitate to submit your own configurations.
I will add them to the package.

Links
-----

* `Yi-Xin Liu's personal website <http://ngpy.org>`_
* `Development version <https://github.com/liuyxpp/mpltex>`_
"""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='mpltex',
    version='0.2',
    license='BSD',
    description='mpltex is a python package for producing publication quality images using matplotlib.',
    author='Yi-Xin Liu',
    author_email='liuyxpp@gmail.com',
    url='https://github.com/liuyxpp/mpltex',
    packages=['mpltex'],
    include_package_data=True,
    zip_safe=False,
    long_description=__doc__,
    platform='linux',
    install_requires=[
        'matplotlib',
        'brewer2mpl',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering',
        'Topic :: Education',
    ]
     )




# -*- coding: utf-8 -*-
"""
acs.py
======

Context decorator for producing figures which is ready to publish
in American Chemical Society.

EPS format for image file are used, because it is high quality and working
in actual physical size rahter than pixel unit.

"""

from .general import MPLdecorator
from .colors import brewer_set1
from .layout import GOLDEN_RATIO

__all__ = ['acs_decorator', ]

# Constants from ACS Authour Guidelines.
width_single_column = 3.25
width_double_column = 7.00

# Default ratio for a single plot figure
# Golden ratio
height_width_ratio = GOLDEN_RATIO  # = height / width

_width = width_single_column
_height = width_single_column * height_width_ratio

_params = {'font.family' : 'serif',
           'font.serif' : ['Times', 'Computer Modern Roman'],
           'font.sans-serif' : ['Helvetica', 'Computer Modern Sans serif'],
           'text.usetex' : True,

           'axes.color_cycle': brewer_set1,
           'axes.labelsize' : 9,
           'axes.linewidth' : 1,

           'figure.figsize' : (_width, _height),
           'figure.subplot.left' : 0.125,
           'figure.subplot.right' : 0.95,
           'figure.subplot.bottom' : 0.1,
           'figure.subplot.top' : 0.95,

           'savefig.dpi' : 300,
           'savefig.format': 'eps',
           #'savefig.bbox': 'tight',
           # this will crop white spaces around images that will make
           # width/height no longer the same as the specified one.

           'legend.fontsize' : 7.5,
           'legend.frameon' : False,
           'legend.numpoints' : 1,
           'legend.handlelength' : 2,
           'legend.scatterpoints' : 1,
           'legend.labelspacing' : 0.5,
           'legend.markerscale' : 0.9,
           'legend.handletextpad' : 0.5,  # pad between handle and text
           'legend.borderaxespad' : 0.5,  # pad between legend and axes
           'legend.borderpad' : 0.5,  # pad between legend and legend content
           'legend.columnspacing' : 1,  # pad between each legend column

           'text.fontsize' : 8,
           'xtick.labelsize' : 8,
           'ytick.labelsize' : 8,

           'lines.linewidth' : 1,
           'lines.markersize' : 4,
           #'lines.markeredgewidth' : 0,
           # 0 will make line-type markers, such as '+', 'x', invisible
          }

acs_decorator = MPLdecorator(_params)

